# -*- encoding: utf-8 -*-
'''
@File        :run.py
@Time        :2021/09/28 09:04:51
@Author        :charles kiko
@Version        :1.0
@Contact        :charles_kiko@163.com
@Desc        :主程序
'''


import argparse
import os
import sys
import configparser
import pandas as pd
import famCircle
import famCircle.bez as bez
from famCircle.filterWGD import filterWGD
from famCircle.hmmer import hmmer
from famCircle.screen import screen
from famCircle.Ks_allocation import Ks_allocation
from famCircle.Ks import Ks
from famCircle.Ks_block import Ks_block
from famCircle.circle import circle
from famCircle.line import line
from famCircle.circle_all import circle_all
from famCircle.circle_family import circle_family

from famCircle.outer import outer
from famCircle.inner import inner

from famCircle.typing import typing
from famCircle.part import part
from famCircle.family_pair import family_pair

from famCircle.tandem import tandem
from famCircle.proximal import proximal
from famCircle.WGD_family import WGD_family
from famCircle.trd import trd
from famCircle.dispersed import dispersed


parser = argparse.ArgumentParser(
    prog = 'famCircle', usage = '%(prog)s [options]', epilog = "", formatter_class = argparse.RawDescriptionHelpFormatter,)
parser.description = '''\
圈图构建
    -------------------------------------- '''
parser.add_argument("-v", "--version", action = 'version', version='0.1.9')

parser.add_argument("-ca", dest = "circle_all",
                    help = "Showing all genetic relationships，the relational file such as,blast, MCScanX, ColinearScan, WGDI are supported;")
parser.add_argument("-l", dest = "line",
                    help = "Showing local genetic relationships,the relational file such as,blast, MCScanX, ColinearScan, WGDI are supported;")
parser.add_argument("-c", dest = "circle",
                    help = "Visualizing Colinearity,the relational file such as,MCScanX、ColinearScan、WGDI;")
parser.add_argument("-ks", dest = "Ks",
                    help = "Computing Ka, Ks of relational genes, the relational file such as,blast, MCScanX, ColinearScan, WGDI are supported;")
parser.add_argument("-ka", dest = "Ks_allocation",
                    help = "Genome-wide KS visualization;")
parser.add_argument("-kb", dest = "Ks_block",
                    help = "block KS visualization;")
parser.add_argument("-f", dest = "filterWGD",
                    help = "WGD")
parser.add_argument("-hmm", dest = "hmmer",
                    help = "Identification of HMM Gene Family;")
parser.add_argument("-fp", dest = "family_pair",
                    help = "blast, MCScanX, ColinearScan, and WGDI were the types of relational files that supported intra-family relational genes;")
parser.add_argument("-cf", dest = "circle_family",
                    help = "Chromosome localization of family members;")
parser.add_argument("-s", dest = "screen",
                    help = "Number and distribution of family members on chromosomes;")
parser.add_argument("-t", dest = "typing",
                    help = "Member data standardization;")
parser.add_argument("-o", dest = "outer",
                    help = "Distribution of family members and inference of repetition time;")
parser.add_argument("-i", dest = "inner",
                    help = "Distribution of family members and inference of repetition time;")
parser.add_argument("-p", dest = "part",
                    help = "Distribution of local family members and inference of repetition time;")
parser.add_argument("-wf", dest = "WGD_family",
                    help = "WGD")
parser.add_argument("-td", dest = "tandem",
                    help = "td")
parser.add_argument("-pd", dest = "proximal",
                    help = "pd")
parser.add_argument("-trd", dest = "trd",
                    help = "trd")
parser.add_argument("-dsd", dest = "dispersed",
                    help = "dsd")


args = parser.parse_args()

def run_filterWGD():
    options = bez.load_conf(args.filterWGD, 'filterWGD')
    filterWGD1 = filterWGD(options)
    filterWGD1.run()

def run_tandem():
    options = bez.load_conf(args.tandem, 'tandem')
    tandem1 = tandem(options)
    tandem1.run()

def run_proximal():
    options = bez.load_conf(args.proximal, 'proximal')
    proximal1 = proximal(options)
    proximal1.run()

def run_trd():
    options = bez.load_conf(args.trd, 'trd')
    trd1 = trd(options)
    trd1.run()

def run_dispersed():
    options = bez.load_conf(args.dispersed, 'dispersed')
    dispersed1 = dispersed(options)
    dispersed1.run()

def run_hmmer():
    options = bez.load_conf(args.hmmer, 'hmmer')
    hmmer1 = hmmer(options)
    hmmer1.run()

def run_screen():
    options = bez.load_conf(args.screen, 'screen')
    screen1 = screen(options)
    screen1.run()

def run_Ks_allocation():
    options = bez.load_conf(args.Ks_allocation, 'Ks_allocation')
    lookKs1 = Ks_allocation(options)
    lookKs1.run()

def run_Ks():
    options = bez.load_conf(args.Ks, 'Ks')
    lookKs0 = Ks(options)
    lookKs0.run()

def run_Ks_block():
    options = bez.load_conf(args.Ks_block, 'Ks_block')
    lookKs0 = Ks_block(options)
    lookKs0.run()

def run_typing():
    options = bez.load_conf(args.typing, 'typing')
    typing1 = typing(options)
    typing1.run()

def run_circle():
    options = bez.load_conf(args.circle, 'circle')
    circle1 = circle(options)
    circle1.run()

def run_line():
    options = bez.load_conf(args.line, 'line')
    circle1 = line(options)
    circle1.run()

def run_circle_all():
    options = bez.load_conf(args.circle_all, 'circle_all')
    circle0 = circle_all(options)
    circle0.run()

def run_WGD_family():
    options = bez.load_conf(args.WGD_family, 'WGD_family')
    WGD_family0 = WGD_family(options)
    WGD_family0.run()

def run_circle_family():
    options = bez.load_conf(args.circle_family, 'circle_family')
    circle0 = circle_family(options)
    circle0.run()

def run_outer():
    options = bez.load_conf(args.outer, 'outer')
    outer1 = outer(options)
    outer1.run()

def run_inner():
    options = bez.load_conf(args.inner, 'inner')
    inner1 = inner(options)
    inner1.run()

def run_part():
    options = bez.load_conf(args.part, 'part')
    inner1 = part(options)
    inner1.run()

def run_family_pair():
    options = bez.load_conf(args.family_pair, 'family_pair')
    family_pair1 = family_pair(options)
    family_pair1.run()

def module_to_run(argument):
    switcher = {
        'filterWGD': run_filterWGD,
        'tandem': run_tandem,
        'proximal': run_proximal,
        'trd': run_trd,
        'dispersed': run_dispersed,
        'WGD_family': run_WGD_family,
        'hmmer': run_hmmer,
        'screen': run_screen,
        'Ks_allocation': run_Ks_allocation,
        'Ks': run_Ks,
        'Ks_block': run_Ks_block,
        'typing': run_typing,
        'circle': run_circle,
        'line': run_line,
        'circle_all': run_circle_all,
        'circle_family': run_circle_family,
        'outer': run_outer,
        'inner': run_inner,
        'part': run_part,
        'family_pair':family_pair,
    }
    return switcher.get(argument)()
    
def main():
    path = famCircle.__path__[0]
    options = {
               'filterWGD': 'filterWGD.conf',
               'tandem': 'tandem.conf',
               'proximal': 'proximal.conf',
               'trd': 'trd.conf',
               'dispersed': 'dispersed.conf',
               'WGD_family': 'WGD_family.conf',
               'hmmer': 'hmmer.conf',
               'screen': 'screen.conf',
               'Ks_allocation': 'Ks_allocation.conf',
               'Ks': 'Ks.conf',
               'Ks_block': 'Ks_block.conf',
               'typing': 'typing.conf',
               'circle': 'circle.conf',
               'line': 'line.conf',
               'circle_all': 'circle_all.conf',
               'circle_family': 'circle_family.conf',
               'outer': 'outer.conf',
               'inner': 'inner.conf',
               'part': 'part.conf',
               'family_pair': 'family_pair.conf',
               }
    for arg in vars(args):
        value = getattr(args, arg)
        # print(value)
        if value is not None:
            if value in ['?', 'help', 'example']:
                f = open(os.path.join(path, 'example', options[arg]))
                print(f.read())
            elif value == 'e':
                out = '''\
        File example
        [fpchrolen]
        chromosomes number_of_bases
        *   *
        *   *
        *   *
        [fpgff]
        chromosomes gene    start   end
        *   *   *   *
        *   *   *   *
        *   *   *   *
        [fpgenefamilyinf]
        gene1   gene2   Ka  Ks
        *   *   *   *
        *   *   *   *
        *   *   *   *
        [alphagenepairs]
        gene1   gene2
        *   *   *
        *   *   *
        *   *   *

        The file columns are separated by Tab
        -----------------------------------------------------------    '''
                print(out)
            elif not os.path.exists(value):
                print(value+' not exits')
                sys.exit(0)
            else:
                module_to_run(arg)




# -*- encoding: utf-8 -*-
'''
@File        :part.py
@Time        :2021/07/11 12:44:00
@Author        :charles kiko
@Version        :1.0
@Contact        :charles_kiko@163.com
@Desc        :家族分布染色体
'''

import csv
import sys
import re
from math import *
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from matplotlib.patches import *
from matplotlib.patches import Circle, Ellipse, Arc
from pylab import *
from famCircle.bez import *

class part():
    def __init__(self, options):
        self.makers = 200
        # self.chrolist = "mes,ssu"
        for k, v in options:
            setattr(self, str(k), v)
            print(k, ' = ', v)

    def read_csv(self):
        matrix_chr1 = pd.read_csv(self.lens, sep='\t', header=None, index_col=0)
        # print(matrix_chr1)
        matrix_gff1 = pd.read_csv(self.gff, sep='\t', header=None, index_col=1)
        new_name_list = []
        for item in matrix_gff1.iterrows():
            # print(item[0])
            old_name = str(item[0])
            # new_name = readname(old_name,self.chro_name)
            new_name = old_name
            new_name_list.append(new_name)
            # matrix_gff1.iloc[old_name,0] = new_name
            # print('#####################')
        # print(new_name_list)
        matrix_gff1.index = new_name_list
        matrix_gff1.groupby(by=0)
        # print(matrix_gff1)

        family_list = []
        for row in open(self.genefamily, 'r'):
            if row != '\n':
                name = row.strip('\n')
                # name = readname(name,self.chro_name)
                chro = str(name.split('^')[0])
                if chro == self.chro_name and name not in family_list:
                    family_list.append(name)

        csv_reader = csv.reader(open(self.pairs_file))
        pair_dic = {}
        for row in csv_reader:
            name1,name2 = readname(str(row[0]),self.chro_name), readname(str(row[1]),self.chro_name)
            chro1, chro2 = str(name1.split('^')[0]),str(name2.split('^')[0])
            name1,name2 = str(row[0]),str(row[1])
            if chro1 != self.chro_name or chro2 != self.chro_name:
                continue
            if name1 > name2:
                name = name1 + '%' + name2
            else:
                name = name2 + '%' + name1
            pair_dic[name] = 0
        return matrix_chr1,matrix_gff1,family_list,pair_dic

    def read_colinearscan(self,file):
        pair = []
        pair_list = []
        with open(file) as f:
            for line in f.readlines():
                line = line.strip()
                # print(line)
                if line[0] == "#":
                    if len(pair_list) != 0:
                        # print(pair_list)#输出
                        pair.append(pair_list)
                    pair_list = []
                    block_dict = {}
                    lt = line.split()
                    block_number = str(lt[2])[:-1]
                    # print(lt)
                    for i in lt:
                        if "=" in i:
                            lt0 = i.split('=')
                            block_dict[str(lt0[0])] = str(lt0[1])
                else:
                    lt = line.split()
                    # name1 = readname(str(lt[0]),self.chro_name)
                    # name2 = readname(str(lt[2]),self.chro_name)
                    lt0 = [str(lt[0]),str(lt[2])]
                    # print(lt0)
                    pair_list.append(lt0)
            if len(pair_list) != 0:
                # print(pair_list)#输出
                pair.append(pair_list)
            # print(pair)
            return pair

    def plot_bez_inner(self, ex1x, ex1y, ex2x, ex2y):
        x = [ex1x, ex1x+((ex2x - ex1x)/3+0.1), ex2x-((ex2x - ex1x)/3+0.1), ex2x]
        y = [ex1y, ex1y+((ex2y - ex1y)/3), ex2y-((ex2y - ex1y)/3), ex2y]
        # ratiox = 0.5
        # ratioy = 1
        # x = [ex1x, ex1x*ratiox, ex2x*ratiox, ex2x]
        # y = [ex1y, ex1y*ratioy, ex2y*ratioy, ex2y]
        # x = [ex1x, ex1x+((ex2x - ex1x)/3), ex2x-((ex2x - ex1x)/3), ex2x]
        # y = [ex1y, ex1y+((ex2y - ex1y)/3), ex2y-((ex2y - ex1y)/3), ex2y]
        step = .01
        t = arange(0, 1+step, step)
        xt = self.Bezier(x, t)# 贝塞尔曲线
        yt = self.Bezier(y, t)
        plot(xt, yt, '-', color='r', lw=.3, alpha=0.3)#alpha 透明度

    def calculate_coef(self,p0, p1, p2, p3):
        c = 3*(p1 - p0)
        b = 3*(p2 - p1) -c
        a = p3 - p0 - c - b
        return c, b, a
    def Bezier(self,plist, t):
        # p0 : origin, p1, p2 :control, p3: destination
        p0, p1, p2, p3 = plist
        # calculates the coefficient values
        c, b, a = self.calculate_coef(p0, p1, p2, p3)
        tsquared = t**2
        tcubic = tsquared*t
        return a*tcubic + b*tsquared + c*t + p0

    def make_plot(self,matrix_chr1,matrix_gff1,family_list,pair_dic):
        minin,maxin = int(self.interval.split(',')[0]),int(self.interval.split(',')[1])
        chro_lingth = int(matrix_chr1.loc[self.chro_name,1])
        chro_order = int(matrix_chr1.loc[self.chro_name,2])
        gene_order = chro_lingth/chro_order
        gene_number = abs(minin-maxin)
        chro_lingth1 = gene_number*gene_order
        maker = 0.8 * ((1500 * gene_order)/chro_lingth1)
        fig1 = plt.figure(num=1, figsize=(10, 10))  # 确保正方形在屏幕上显示一致，固定figure的长宽相等
        axes1 = fig1.add_subplot(1, 1, 1)
        plt.xlim((0, 1))
        plt.ylim((0, 1))
        self.map1(axes1,maker,0.1,0.3,0.8,'k')# 长染色体0.8
        # self.map1(axes1,maker2,0.1+((0.8-x2)/2),0.65,x2,'k')# 短染色体
        plt.text(0.45, 0.28, self.chro_name)
        plt.axis('off')
        self.pair_index(axes1,matrix_chr1,matrix_gff1,family_list,pair_dic)
        plt.savefig(self.savefile,dpi=1000)

    def map1(self,axes1,maker,x,y,x1,color):
        # makers = 15 #标尺
        # x = 0.1 #起始坐标x
        # y = 0.1 #起始坐标y
        # x1 = 0.8#染色体长度
        # color='k'
        lw = 1  #比例线宽
        y1 = 0.001
        w = 0.01
        alpha = 1
        # print(y+w+y1, x, x+x1,y, x, x+x1, lw)
        plt.axhline(y=y, xmin=x, xmax=x+x1, lw=lw, c=color, alpha=alpha)
        plt.axhline(y=y+w+y1, xmin=x, xmax=x+x1, lw=lw, c=color, alpha=alpha)
        for i in range(int(0.8/maker)+1):
            mx = maker*i
            plt.axvline(x=0.1+mx, ymin=y, ymax=y+(w/2.5), lw=lw, c=color, alpha=alpha)
        base1 = Arc(xy=(x, y+((w+y1)/2)),    # 椭圆中心，（圆弧是椭圆的一部分而已）
                width=w+y1,    # 长半轴
                height=w+y1,    # 短半轴
                angle=90,    # 椭圆旋转角度（逆时针） 
                theta1=0,    # 圆弧的起点处角度
                theta2=180,    # 圆度的终点处角度
                color=color,
                alpha=alpha,
                linewidth=lw
                )
        base2 = Arc(xy=(x1+x, y+((w+y1)/2)),    # 椭圆中心，（圆弧是椭圆的一部分而已）
                width=w+y1,    # 长半轴
                height=w+y1,    # 短半轴
                angle=-90,    # 椭圆旋转角度（逆时针） 
                theta1=0,    # 圆弧的起点处角度
                theta2=180,    # 圆度的终点处角度
                color=color,
                alpha=alpha,
                linewidth=lw   #线宽像素
                )
        axes1.add_patch(base1)
        axes1.add_patch(base2)

    def map2(self,axes1,x,y,x1,color):
        # makers = 15 #标尺
        # x = 0.1 #起始坐标x
        # y = 0.1 #起始坐标y
        # x1 = 0.8#染色体长度
        # color='k'
        lw = 2  #比例线宽
        alpha = 1
        # print(y+w+y1, x, x+x1,y, x, x+x1, lw)
        plt.axhline(y=y, xmin=x, xmax=x+x1, lw=lw, c=color, alpha=alpha)

    def transform_pt2(self, rad, r):
        return r*cos(rad), r*sin(rad)

    def plot_bez_Ks2(self, ex1x, ex1y, ex2x, ex2y, col, ratio,gene1, gene2):
    #    print "bez Ks 2"
        # print('rad1, r1',rad1, r1)
        # ex1x, ex1y = self.transform_pt2(rad1, r1)
        # ex2x, ex2y = self.transform_pt2(rad2, r2)
        # ratio = -0.7#0.5
        sita = pi / 2
        if ex1x != ex2x:
            sita = atan((ex2y-ex1y)/(ex2x-ex1x))
        d = sqrt((ex2x-ex1x)**2+(ex2y-ex1y)**2)
        L = d * ratio
        P1x = ex1x + L*sin(sita)
        P1y = ex1y - L*cos(sita)
        P2x = ex2x + L*sin(sita)
        P2y = ex2y - L*cos(sita)
        step = .01
        t = arange(0, 1+step, step)
        x=[ex1x, P1x, P2x, ex2x]
        y=[ex1y, P1y, P2y, ex2y]
        # print('x,y,t',x,y,t)
        xt = Bezier(x,t)
        yt = Bezier(y,t)
        xx3 = xt[int(len(xt)/2)]
        yy3 = yt[int(len(yt)/2)]-0.001
        s = '<-'+str(gene1)+'-'+ str(gene2)+'->'
        text(xx3,yy3, s = str(s),color=col, fontsize = 1.5, rotation=30, alpha=0.8)# 添加基因标注
        plot(xt, yt, '-', color = col, lw = 0.3)#0.1

    def classify(self, start1, start2, grade):
        colorlist = ['red', 'lime', 'orange', 'chocolate', 'yellow', 'palegreen', 'teal', 'dodgerblue', 'blueviolet', 'fuchsia']
        # lt = self.ks_concern.strip('\n').split(',')
        # length = len(lt)
        color = colorlist[grade]
        class1 = grade
        return color, class1

    def pair_index(self,axes1,matrix_chr1,matrix_gff1,family_list,pair_dic):
        minin,maxin = int(self.interval.split(',')[0]),int(self.interval.split(',')[1])
        chro_lingth = int(matrix_chr1.loc[self.chro_name,1])
        chro_order = int(matrix_chr1.loc[self.chro_name,2])
        gene_order = chro_lingth/chro_order
        gene_number = abs(minin-maxin)
        chro_lingth1 = gene_number*gene_order
        maker = 0.8 * ((1500 * gene_order)/chro_lingth1)
        indexx = 0.8/chro_lingth
        inde3 = minin
        name_length = len(str(family_list[0]))-len(str(self.chro_name))-1
        while 1:
            i = self.chro_name + '^' +str(inde3).zfill(name_length)
            # print(i)
            try:
                start0 = max(int(matrix_gff1.loc[i,2]),int(matrix_gff1.loc[i,3]))
                break
            except:
                # print(inde3)
                inde3 = inde3 + 1


        inde3 = maxin
        while 1:
            if inde3 >= chro_order:
                start9 = chro_lingth
                break
            i = self.chro_name + '^' +str(inde3).zfill(name_length)
            # print(i)
            try:
                start9 = min(int(matrix_gff1.loc[i,2]),int(matrix_gff1.loc[i,3]))
                break
            except:
                # print(inde3)
                inde3 = inde3 + 1

        # print(chro_lingth,chro_order)
        # laststart = start
        gene_list = []
        gene_index_dic = {}
        shiftlevel = 0
        gene2shift = {}
        gene2pos = {}
        gene2location = {}
        for i in family_list:
            # print(i)
            if str(i.split('^')[0]) != self.chro_name:
                continue
            # print(int(matrix_gff1.loc[i,5]) , minin , int(matrix_gff1.loc[i,5]) , maxin)
            if int(matrix_gff1.loc[i,2]) < start0 or int(matrix_gff1.loc[i,3]) > start9:
                continue
            gene_list.append(i)
            start = min(int(matrix_gff1.loc[i,2]),int(matrix_gff1.loc[i,3]))
            gene_index_dic[i] = 0.8*((start-start0)/(chro_lingth-start0))
            gene2pos[i] = start
            # print(start)
        # print(gene_list)
        gene_list.sort()
        start = gene_index_dic[str(gene_list[0])]
        gene2location[str(gene_list[0])] = start
        gene2shift[str(gene_list[0])] = shiftlevel
        laststart = start
        shiftlist = []
        space_ = 0

        for i in gene_list:
            start = gene_index_dic[str(i)]
            # print(start-laststart)
            if(start-laststart < float(self.space)):
                shiftlevel = shiftlevel + 1
                space_ = space_ + (start-laststart)
                shiftlist.append(space_)
            else:
                shiftlevel=0
            self.map2(axes1,start+0.1,(shiftlevel-1)*0.005+0.32+(shiftlevel-1)*0.001,0.002,'blue')
            gene2location[i] = start
            gene2shift[i] = shiftlevel
            laststart = start
        gene_size_ = max(shiftlist)
        ks = read_ks0(self.ks)
        for genepair in pair_dic.keys():
            # print(genepair)
            gene1, gene2 = str(genepair.split('%')[0]),str(genepair.split('%')[1])
            if gene1 not in gene2location.keys() or gene2 not in gene2location:
                continue# 此处还要修改
            start1 = gene2location[gene1]
            shift1 = gene2shift[gene1]
            start2 = gene2location[gene2]
            shift2 = gene2shift[gene2]
            if abs(start1 - start2) > gene_size_:
                continue
            if gene1 + str('_') + gene2 in ks:
                ks_v = ks[gene1 + str('_') + gene2]
                color = return_col(Ks_v,float(self.ks_concern.split(',')[0]),float(self.ks_concern.split(',')[1]))
            if gene2 + str('_') + gene1 in ks:
                ks_v = ks[gene1 + str('_') + gene2]
                color = return_col(Ks_v,float(self.ks_concern.split(',')[0]),float(self.ks_concern.split(',')[1]))
            else:
                color = 'block'
            print(color)
            ratio = 0.5
            if (color != None):
                # print(start1, start2)
                if (self.clusters == "True"):
                    self.plot_bez_Ks2(start1+0.101, (shift1-1) * 0.006+0.32, start2+0.101, (shift2-1)*0.006+0.32, color, ratio,gene1, gene2)
                else:
                    self.plot_bez_Ks2(start1+0.101, (shift1-1) * 0.006+0.32, start2+0.101, (shift2-1)*0.006+0.32, 'red', ratio,gene1, gene2)
            else :
                # print('间距不达标')
                pass
    def run(self):
        matrix_chr1,matrix_gff1,family_list,pair_dic = self.read_csv()
        # pair = self.read_colinearscan(self.pairs_file)
        self.make_plot(matrix_chr1,matrix_gff1,family_list,pair_dic)



# -*- encoding: utf-8 -*-
'''
@File        :inner.py
@Time        :2021/09/28 11:21:41
@Author        :charles kiko
@Version        :1.0
@Contact        :charles_kiko@163.com
@Desc        :内卷的圈图
'''


import re
import os
import sys
from math import *
import csv
import pandas as pd
from matplotlib.patches import *
from pylab import *

# from io import BytesIO
# import base64
# from lxml import etree

from famCircle.bez import *


class inner():
    def __init__(self, options):
        self.genessorted = []
        self.file_type = "coliearity"
        self.geneno = 0
        self.genepair2Ks = {}
        self.genepair2Ka = {}
        ##### circle parameters
        self.GAP_RATIO = 4 #gaps between chromosome circle, chr:gap = 4: 1
        self.radius = 0.33
        # figurefile = "_".join(sys.argv[1:len(sys.argv)])+".genefam"
        #### gene block parameters
        self.space = "0.01" # radian 0.01
        self.peripheral = 'False'
        self.block= 0.005 #block scize
        self.blockthick = 0.004 #0.006
        self.shiftratio = 1.1 # define the distance between overlapping glocks
        self.clusters = "True"
        self.specieslist = []
        self.iscompletegenome = {}
        self.chro2len = {}
        self.vvchrolist = []
        self.otherchrolist = []
        self.labels = []
        self.start_list = []
        self.gene2pos={}
        self.gene2chain = {}
        self.orderlist = {}
        self.genes = []
        self.genepair0 = []
        for k, v in options:
            setattr(self, str(k), v)
            print(k, ' = ', v)

    # def ks(self, figurefile, fpchrolen, fpgff, fpgenefamilyinf, chrolist):
    def ksrun(self):
        # print('kaishi')
        figure(1, (8, 8))  ### define the a square, or other rectangle of the figure, if to produce an oval here
        root =axes([0, 0, 1, 1])
        if self.species1 != self.species2:
            # fpgenefamilyinf = open(self.ks, 'r', encoding='utf-8')
            genefamily = open(self.genefamily, 'r', encoding='utf-8')

            fpchrolen1 = open(self.lens1, 'r', encoding='utf-8')
            for row in fpchrolen1:
                # print('ceshi')
                chro,length,order = row.split('\t')[0],row.split('\t')[1],row.split('\t')[2]
                self.labels.append(chro)
                self.chro2len[chro] = int(length)
                self.orderlist[chro] = int(order)
                # self.otherchrolist.append(chro)
            ## input gff
            fpchrolen1.close()
            fpchrolen2 = open(self.lens2, 'r', encoding='utf-8')
            for row in fpchrolen2:
                # print('ceshi')
                chro,length,order = row.split('\t')[0],row.split('\t')[1],row.split('\t')[2]
                self.labels.append(chro)
                self.chro2len[chro] = int(length)
                self.orderlist[chro] = int(order)
                # self.otherchrolist.append(chro)
            ## input gff
            fpchrolen2.close()


            familylist = []

            for row in genefamily:
                if (row != '\n'):
                    familyname = readname(row.strip('\n').split()[0],self.species1+self.species2)
                    chro = str(familyname.split('^')[0])
                    if chro not in self.chro2len.keys():
                        continue
                    familylist.append(familyname)
                    self.genepair0.append(familyname)
            genefamily.close()

            fpgff1 = open(self.gff1, 'r', encoding='utf-8')
            for row in fpgff1:
                ch, gene, start, end = row.split()[0],row.split()[1],row.split()[2],row.split()[3]    #mrna, , software
                gene = readname(gene,self.species1+self.species2)
                if (gene in familylist):
                    start = int(start)
                    end = int(end)
                    self.gene2pos[gene] = int(start)# 基因位置
                    self.gene2chain[gene] = int((end-start)/abs(end -start))# 基因正负
                else:
                    pass
            fpgff1.close()
            fpgff2 = open(self.gff2, 'r', encoding='utf-8')
            for row in fpgff2:
                ch, gene, start, end = row.split()[0],row.split()[1],row.split()[2],row.split()[3]    #mrna, , software
                gene = readname(gene,self.species1+self.species2)
                if (gene in familylist):
                    start = int(start)
                    end = int(end)
                    self.gene2pos[gene] = int(start)# 基因位置
                    self.gene2chain[gene] = int((end-start)/abs(end -start))# 基因正负
                else:
                    pass
            fpgff2.close()


        else:
            fpchrolen = open(self.lens1, 'r', encoding='utf-8')
            fpgff = open(self.gff1, 'r', encoding='utf-8')
            
            genefamily = open(self.genefamily, 'r', encoding='utf-8')
            for row in fpchrolen:
                # print('ceshi')
                chro,length,order = row.split('\t')[0],row.split('\t')[1],row.split('\t')[2]
                self.labels.append(chro)
                self.chro2len[chro] = int(length)
                self.orderlist[chro] = int(order)
                # self.otherchrolist.append(chro)
            ## input gff
            familylist = []
            for row in genefamily:
                if (row != '\n'):
                    familyname = readname(row.strip('\n').split()[0],self.species1)
                    chro = str(familyname.split('^')[0])
                    if chro not in self.chro2len.keys():
                        continue
                    familylist.append(familyname)
                    self.genepair0.append(familyname)
            genefamily.close()

            for row in fpgff:
                ch, gene, start, end = row.split()[0],row.split()[1],row.split()[2],row.split()[3]    #mrna, , software
                gene = readname(gene,self.species1)
                if (gene in familylist):
                    start = int(start)
                    end = int(end)
                    self.gene2pos[gene] = int(start)# 基因位置
                    self.gene2chain[gene] = int((end-start)/abs(end -start))# 基因正负
                else:
                    pass
            fpgff.close()


        ### input gene family gene1 gene2 Ka Ks
        fpgenefamilyinf = open(self.ks, 'r', encoding='utf-8')
        for row in fpgenefamilyinf:
            if (row.split()[0] == 'id1' or len(row.split()) < 4):
                continue
            gene1, gene2, Ka, Ks = row.split()[0],row.split()[1],row.split()[2],row.split()[3]
            gene1 = readname(gene1,self.species1+self.species2)
            gene2 = readname(gene2,self.species1+self.species2)
            if (gene1 not in self.gene2pos.keys() or gene2 not in self.gene2pos.keys()):
                continue
            genepair = ""
            # print(type(gene1))
            if gene1 < gene2:
               genepair = gene1 + " " + gene2
            else:
               genepair = gene2 + " " + gene1
            if len(gene1.split("^")[0]) < 10 and len(gene2.split("^")[0]) < 10:#数字筛选chr名字长度
               self.genepair2Ks[genepair] = float(Ks)
        fpgenefamilyinf.close()
        # print("genes ", self.genes, "genes")
        return root

    def rad_to_coord(self,angle, radius):
        return radius*cos(angle), radius*sin(angle)

    def to_deg(self,bp, total):
        # from basepair return as degree 从碱基对返回为学位
        return bp*360./total

    def to_radian(self,bp, total):# 返回弧度
        return radians(bp*360./total)

    def plot_arc(self,start, stop, radius):
        # start, stop measured in radian
        #print start, stop
        t = arange(start, stop, pi/720.)
        x, y = radius*cos(t), radius*sin(t)
        plot(x, y, "k-", alpha=.5)

    def plot_cap(self,angle, clockwise=True):
        radius=self.sm_radius
        # angle measured in radian, clockwise is boolean
        if clockwise: t = arange(angle, angle+pi, pi/30.)
        else: t = arange(angle, angle-pi, -pi/30.)
        x, y = radius*cos(t), radius*sin(t)
        middle_r = (self.radius_a+self.radius_b)/2
        x, y = x + middle_r*cos(angle), y + middle_r*sin(angle)
        plot(x, y, "k-", alpha=.5)

    def plot_arc_block1(self,start, chain, radius, end):
        t = arange(start, start+end, pi/720.)
        x,y = radius * cos(t), radius*sin(t)
        plot(x, y, "g-", alpha=0.5)

    def plot_arc_block(self,start, chain, radius):
        t = arange(start, start+self.block, pi/720.)
        x,y = radius * cos(t), radius*sin(t)
        plot(x, y, "b-", alpha=0.5)

    def zj(self):
        fullchrolen = int(pd.DataFrame(self.chro2len.values()).sum())
        chr_number = len(self.labels) # total number of chromosomes
        GAP = fullchrolen/self.GAP_RATIO/chr_number # gap size in base pair
        total_size = fullchrolen + chr_number * GAP # base pairs
        for i in range(chr_number):
            self.start_list.append(0)
        # self.start_list = [0]*chr_number
        for i in range(1, chr_number):
            self.start_list[i] = self.start_list[i-1] + self.chro2len[self.labels[i-1]] + GAP
        stop_list = [(self.start_list[i] + self.chro2len[self.labels[i]]) for i in range(chr_number)]
        # print("start list", self.start_list)
        return stop_list, total_size, chr_number

    def transform_deg(self,ch, pos, total_size):
        return self.to_deg(pos + self.start_list[ch], total_size)

    def transform_pt(self,ch, pos, r, total_size):
        # convert chromosome position to axis coords
        # print("transform", ch, pos, r)
    #    print "startlist", self.start_list[ch]
        rad = self.to_radian(pos + self.start_list[ch], total_size)
        return r*cos(rad), r*sin(rad)

    def transform_pt1(self,ch, pos, r, total_size):
        # convert chromosome position to axis coords
        # print("transform", ch, pos, r)
    #    print "startlist", self.start_list[ch]
        rad = self.to_radian(pos + self.start_list[ch], total_size)
        return r*cos(rad), r*sin(rad),rad

    def transform_pt2(self,rad, r):
        return r*cos(rad), r*sin(rad)

    def plot_scale(self,length,number):
        plot([0.5-length-0.05,0.5-0.05],[-0.45,-0.45], color='#333631',linestyle='-', lw = 1.5)
        plot([0.5-length-0.05,0.5-length-0.05],[-0.45,-0.44], color='red',linestyle='-', lw = 1)
        plot([0.5-0.05,0.5-0.05],[-0.45,-0.44], color='#333631',linestyle='-', lw = 1)
        text(0.5-length-0.05+(length/2)-0.02,-0.43, s = str(number),color='black', fontsize = 8, alpha=0.8)# 添加基因标注

    def plot_bez_inner(self,p1, p2, cl, total_size):
    #    print "inner"
        a, b, c = p1
        ex1x, ex1y = self.transform_pt(a, b, c, total_size)
        a, b, c = p2
        ex2x, ex2y = self.transform_pt(a, b, c, total_size)
        # Bezier ratio, controls curve, lower ratio => closer to center
        ratio = .5
        x = [ex1x, ex1x*ratio, ex2x*ratio, ex2x]
        y = [ex1y, ex1y*ratio, ex2y*ratio, ex2y]
        step = .01
        t = arange(0, 1+step, step)
        xt = Bezier(x, t)
        yt = Bezier(y, t)
        plot(xt, yt, '-', color=cl, lw=.1)

    def plot_bez_Ks2(self,rad1, r1, rad2, r2, col, ratio, gene1, gene2):
        ex1x, ex1y = self.transform_pt2(rad1, r1)
        ex2x, ex2y = self.transform_pt2(rad2, r2)
        sita = pi/2
        if ex1x != ex2x:
            sita = atan((ex2y-ex1y)/(ex2x-ex1x))
        d = sqrt((ex2x-ex1x)**2+(ex2y-ex1y)**2)
        L = d * ratio
        P1x = ex1x + L*sin(sita)
        P1y = ex1y - L*cos(sita)
        P2x = ex2x + L*sin(sita)
        P2y = ex2y - L*cos(sita)
        step = .01
        t = arange(0, 1+step, step)
        x=[ex1x, P1x, P2x, ex2x]
        y=[ex1y, P1y, P2y, ex2y]
        xt = Bezier(x,t)
        yt = Bezier(y,t)
        # i = int(len(xt)/2)
        # xx,yy = xt[i],yt[i]
        # # print(xx,yy)
        # buoy = '<-' + str(gene1) + '-' + str(gene2) + '->'
        # text(xx,yy, s = buoy,color=col, fontsize = 1,alpha=0.8)# 添加基因标注
        plot(xt, yt, '-', color = col, lw = 0.1)

    def run_plot(self, Ks, start1, start2, grade):
        colorlist = ['red', 'lime', 'orange', 'chocolate', 'yellow', 'palegreen', 'teal', 'dodgerblue', 'blueviolet', 'fuchsia']
        lt = self.ks_concern.strip('\n').split(',')
        length = len(lt)
        if (Ks > eval(lt[1]) or Ks < eval(lt[0])):
            color = 'gray'
            class1 = 'None'
            if self.peripheral == 'True':
                return color, class1
            else:
                return None,None
        else:
            color = colorlist[grade]
            class1 = grade
            return color, class1

    def run(self):
        self.radius_a = float(self.radius)
        self.radius_b = self.radius_a + 0.005
        self.sm_radius=(self.radius_b-self.radius_a)/2 #telomere capping
        if (os.path.exists(self.savecsv)):
            os.remove(self.savecsv)
        lt = ['gene1', 'gene2', 'start1', 'shift1', 'start2', 'shift2', 'class1']
        with open(self.savecsv, "a", newline='', encoding='utf-8') as file:
            writer = csv.writer(file ,delimiter=',')
            writer.writerow(lt)
        root = self.ksrun()
        stop_list, total_size, chr_number = self.zj()
        gene2pos = self.gene2pos
        ## sort gene according to lacation on circle

        for i in range(len(self.genepair0)):
            if self.geneno == 0:
                self.genessorted.append(self.genepair0[0])
                self.geneno = self.geneno + 1
            else:
                firstgene = self.genessorted[0]
                # print(firstgene)
                lastgene = self.genessorted[-1]
                chf = firstgene.split("^")[0]
                chl = lastgene.split("^")[0]
             
                # print(firstgene, lastgene, chf, chl, gene2pos[firstgene])
                posf = gene2pos[firstgene] + self.start_list[self.labels.index(chf)]
                posl = gene2pos[lastgene] + self.start_list[self.labels.index(chl)]
                chi = self.genepair0[i].split("^")[0]
                posi = gene2pos[self.genepair0[i]] + self.start_list[self.labels.index(chi)]
        #       print posf, posl, posi
                if posi <= posf:
                   self.genessorted[0:0] = [self.genepair0[i]]
                elif posi >= posl:
                    self.genessorted.append(self.genepair0[i])
                else:
                    for j in range(len(self.genessorted)-1):
                        chj = self.genessorted[j].split("^")[0]
                        posj = gene2pos[self.genessorted[j]] + self.start_list[self.labels.index(chj)]
                        chj1 = self.genessorted[j+1].split("^")[0]
                        posj1 = gene2pos[self.genessorted[j+1]]+self.start_list[self.labels.index(chj1)]
                        # print(posj, posj1, posi)
                        if posi > posj and posi < posj1:
                            self.genessorted[j+1:j+1] = [self.genepair0[i]]
                # print("genesort ", self.genessorted)

        # the chromosome layout
        j = 0
        for start, stop in zip(self.start_list, stop_list):# 绘制染色体
            start, stop = self.to_radian(start, total_size), self.to_radian(stop, total_size)
            # shaft
            self.plot_arc(start, stop, self.radius_a)# 染色体内圈
            self.plot_arc(start, stop, self.radius_b)# 染色体外圈
            # telemere capping
            self.plot_cap(start, clockwise=False)# 右鸭舌部分
            self.plot_cap(stop)# 左鸭舌部分
            # chromosome self.labels
            label_x, label_y = self.rad_to_coord((start+stop)/2, self.radius_b*1.1)
            text(label_x, label_y, self.labels[j], horizontalalignment="center", verticalalignment="center")
            j += 1

        ### Again: define shift level and draw gene blocks
        shiftlevel = 0
        gene2shift = {}
        gene2location = {}
        cho = self.genessorted[0].split("^")[0]
        pos0 = self.gene2pos[self.genessorted[0]] + self.start_list[self.labels.index(cho)]
        chain = self.gene2chain[self.genessorted[0]]
        start = self.to_radian(pos0, total_size)
        # self.plot_arc_block(start, chain, self.radius_a - shiftlevel * self.shiftratio * self.blockthick)
        #gene2location[self.genessorted[0]]= start
        #gene2shift[self.genessorted[0]] = shiftlevel
        laststart = start
        shiftlist = []
        space_ = 0
        for i in range(len(self.genessorted)):
            if self.genessorted[i] not in self.genepair0:
                continue
            chi = self.genessorted[i].split("^")[0]
            posi = self.gene2pos[self.genessorted[i]] + self.start_list[self.labels.index(chi)]
            chain = self.gene2chain[self.genessorted[i]]
            start = self.to_radian(posi, total_size)
            if(start-laststart < float(self.space)):
                shiftlevel = shiftlevel + 1
                space_ = space_ + (start-laststart)
                shiftlist.append(space_)
            else:
                 shiftlevel=0
                 space_ = 0
            shiftradius = self.radius_a - shiftlevel * self.shiftratio * self.blockthick
        #    print "draw block: ", start, chain, shiftlevel, self.shiftratio, self.blockthick, shiftradius, self.radius_a
            self.plot_arc_block(start, chain, self.radius_a - (shiftlevel+1) * self.shiftratio * self.blockthick)
            laststart = start

        gene_size_ = max(shiftlist)

        ### define shift level and draw gene blocks
        shiftlevel = 0
        gene2shift = {}
        gene2location = {}
        cho = self.genessorted[0].split("^")[0]
        pos0 = self.gene2pos[self.genessorted[0]] + self.start_list[self.labels.index(cho)]
        chain = self.gene2chain[self.genessorted[0]]
        # print('pos0,total_size:',pos0, total_size)
        start = self.to_radian(pos0, total_size)
        self.plot_arc_block1(start, chain, self.radius_a - shiftlevel * self.shiftratio * self.blockthick, gene_size_)
        gene2location[self.genessorted[0]]= start
        gene2shift[self.genessorted[0]] = shiftlevel
        laststart = start

        for i in range(len(self.genessorted)):
            if self.genessorted[i] not in self.genepair0:
                continue
            chi = self.genessorted[i].split("^")[0]
            posi = self.gene2pos[self.genessorted[i]] + self.start_list[self.labels.index(chi)]
            chain = self.gene2chain[self.genessorted[i]]
            # print('pos0,total_size:',pos0, total_size)# pos0,total_size: 2120381 466090045.0
            start = self.to_radian(posi, total_size)
            if(start-laststart < float(self.space)):
                shiftlevel = shiftlevel + 1
            else:
                 shiftlevel=0
            shiftradius = self.radius_a - shiftlevel * self.shiftratio * self.blockthick

        #    print "draw block: ", start, chain, shiftlevel, self.shiftratio, self.blockthick, shiftradius, self.radius_a
            gene2location[self.genessorted[i]] = start
            gene2shift[self.genessorted[i]] = shiftlevel
            laststart = start
        ### draw Ks lines between genes
        # print("***************************************************")
        if self.file_type == 'famCircle':
            pair_list = read_famCircle(self.pair_file,self.species1)
        elif self.file_type == 'coliearity':
            pair_list = read_coliearity0(self.pair_file,self.species1)
        elif self.file_type == 'BLAST':
            pair_list = read_blast(self.pair_file,self.species1)
        else:
            print("Exception string: File is of unknown type")
            exit()
        for genepair in pair_list:
            if genepair not in self.genepair2Ks.keys():
                continue
        # for genepair in self.genepair2Ks.keys():
        #    print genepair
            gene1, gene2 = genepair.split(' ')
            Ks = self.genepair2Ks[genepair]
            if (gene1 not in gene2location.keys() or gene2 not in gene2location.keys()):
                # print(gene1)
                continue
            start1 = gene2location[gene1]
            # print(start1)
            shift1 = gene2shift[gene1]
            start2 = gene2location[gene2]
            # print(start2)
            shift2 = gene2shift[gene2]
            # print ("start", start1, start2)# start 6.015783710157899 6.133748376342092
            if abs(start1 - start2) > gene_size_:
                continue
            space_ks = (abs(float(self.ks_concern.strip('\n').split(',')[1]) - float(self.ks_concern.strip('\n').split(',')[0])))/3
            grade = int(Ks / space_ks)# 分级
            color = return_col(Ks,float(self.ks_concern.split(',')[0]),float(self.ks_concern.split(',')[1]))
            ratio = 0.5
            if (color != None):
                if (self.clusters == "True"):
                    lt = [gene1, gene2, start1, shift1, start2, shift2]
                    with open(self.savecsv, "a", newline='', encoding='utf-8') as file:
                        writer = csv.writer(file ,delimiter=',')
                        writer.writerow(lt)
                    self.plot_bez_Ks2(start1, self.radius_a - (shift1+1) * self.shiftratio * self.blockthick, start2, self.radius_a - (shift2+1) * self.shiftratio * self.blockthick, color, ratio, gene1, gene2)
                else:
                    self.plot_bez_Ks2(start1, self.radius_a - (shift1+1) * self.shiftratio * self.blockthick, start2, self.radius_a - (shift2+1) * self.shiftratio * self.blockthick, 'red', ratio, gene1, gene2)
                    lt = [gene1, gene2, start1, shift1, start2, shift2]
                    with open(self.savecsv, "a", newline='', encoding='utf-8') as file:
                        writer = csv.writer(file ,delimiter=',')
                        writer.writerow(lt)
        #### draw tick showing scale of chromosomes
        order_length = (gene_length_order(self.lens1)+gene_length_order(self.lens2))/2
        angle2 = 0
        angle0,angle1 = 0,0
        # print(order_length)
        for i in range(chr_number):
            pos = 0
            # print(self.orderlist[self.labels[i]])
            while pos < self.orderlist[self.labels[i]]:
                # print(pos)
                if pos == 0 :
                    color = "red"
                else:
                    color = 'blue'
                index_order = pos * order_length
                xx1, yy1 = self.transform_pt(i, int(index_order), self.radius_b-0.001, total_size)
                xx2, yy2 = self.transform_pt(i, int(index_order), self.radius_b-0.004, total_size)
                if pos == 1500:
                    xx3, yy3, angle0 = self.transform_pt1(i, int(index_order), self.radius_b-0.003, total_size)
                elif pos == 3000:
                    xx3, yy3, angle1 = self.transform_pt1(i, int(index_order), self.radius_b-0.003, total_size)
                    angle2 = abs(angle0 - angle1)
                # xx3, yy3, angle0 = self.transform_pt1(i, int(index_order), self.radius_b+0.006, total_size)
                # if xx3 > 0:
                #     xx3 = xx3 - 0.005
                #     yy3 = yy3 - 0.005
                # if xx3 < 0:
                #     xx3 = xx3 - 0.006
                #     yy3 = yy3 - 0.006
                # # print('xx3, yy3, angle0, pi/2 - angle0',xx3, yy3, (pi/2)-angle0)
                # angle1 = ((angle0 - (pi/2))/(2*pi))*360
                plot([xx1, xx2], [yy1, yy2], color=color,linestyle='-', lw = .5)
                # text(xx3,yy3, s = str(pos),color='b', fontsize = 5, rotation=angle1, alpha=0.8)# 添加基因标注
                pos = pos + 1500
        self.plot_scale(angle2*self.radius_b,1500)
        #
        root.set_xlim(-.5, .5)
        root.set_ylim(-.5, .5)
        root.set_axis_off()
        savefig(self.savefile, dpi=1000)

        sys.exit(0)



# -*- coding: UTF-8 -*-

from setuptools import find_packages, setup

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

required = ['pandas', 'numpy', 'biopython', 'matplotlib', 'scipy','interval','seaborn>=0.10.1','tqdm']

setup(
    name="famCircle",
    version="0.1.9",
    author="charles_kiko",
    author_email="charles_kiko@163.com",
    description="famCircle",
    license="BSD License",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    package_data={'': ['*.conf','*.ini', '*.csv']},
    classifiers=[
        "Intended Audience :: Science/Research",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
    ],
    entry_points={
        'console_scripts': [
            'famCircle = famCircle.run:main',
        ]
    },
    zip_safe=True,
    install_requires=required
)


# -*- encoding: utf-8 -*-
'''
@File        :circle.py
@Time        :2021/09/28 11:24:46
@Author        :charles kiko
@Version        :1.0
@Contact        :charles_kiko@163.com
@Desc        :圈图
'''


import re
import sys
from math import *

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from matplotlib.patches import *
from matplotlib.patches import Circle, Ellipse
from pylab import *

from famCircle.bez import *


class circle():
    def __init__(self, options):
    ##### circle parameters
        self.GAP_RATIO = 4 #gaps between chromosome circle, chr:gap = 4: 1
        self.radius = 0.33
        self.block=0.01 #block scize
        self.blockthick = 0.004 #0.006
        self.shiftratio = -2.1 # define the distance between overlapping glocks
        self.specieslist = []
        self.iscompletegenome = {}
        self.gene2pos={}
        self.gene2chain = {}
        self.chro2len = {}
        self.otherchrolist = []
        self.labels = []
        self.genes = []
        self.genepairsfile_type = "wgdi"
        self.genepair2Ks = {}
        self.genepair2Ka = {}
        self.block = '0'
        self.class1 = True
        self.start_list = []
        for k, v in options:
            setattr(self, str(k), v)
            print(k, ' = ', v)


    def ksrun(self):
        fpgenefamilyinf = open(self.ks, 'r', encoding='utf-8')
        alphagenepairs = open(self.genepairs, 'r', encoding='utf-8')
        #### gene block parameters
        figure(1, (8, 8))  ### define the a square, or other rectangle of the figure, if to produce an oval here
        root =axes([0, 0, 1, 1])
        if self.species1 != self.species2:
            lengthset = set()
            for i in [self.species1,self.species2]:
                lengthset.add(len(i))
            chrolist = []


            fpchrolen1 = open(self.lens1,'r', encoding='utf-8')
            for row in fpchrolen1:
                chro = row.split('\t')[0]
                for i in lengthset:
                    if (chro[:i] in [self.species1,self.species2]):
                        chrolist.append(chro)
                    else:
                        pass
            fpchrolen1.close()
            fpchrolen2 = open(self.lens2,'r', encoding='utf-8')
            for row in fpchrolen2:
                chro = row.split('\t')[0]
                for i in lengthset:
                    if (chro[:i] in [self.species1,self.species2]):
                        chrolist.append(chro)
                    else:
                        pass
            fpchrolen2.close()


            for i in range(len(chrolist)):
                string = chrolist[i]
            #   print string[0:2]
                isnew = 1
                for sp in self.specieslist:
                    if sp == string[0:2]:
                        isnew = 0
                        break
                if isnew==1:
                    self.specieslist.append(string[0:2])
                    if string == string[0:2]:
                        self.iscompletegenome[string[0:2]] = 1
                    else:
                        self.iscompletegenome[string[0:2]] = 0

            ### input chromosome length
            fpchrolen1 = open(self.lens1,'r', encoding='utf-8')
            for row in fpchrolen1:
                if row[0] == '#' or row == '\n':
                    continue
                chro,length = row.split('\t')[0],row.split('\t')[1]
                # chro = chro.upper()
                if len(chro) > 10 :
                    continue
                # sp = chro[:2].upper()
                sp = chro[:2]
                if self.iscompletegenome[sp] == 1 :
                    self.chro2len[chro] = int(length)
                    self.otherchrolist.append(chro)
                else:
                    if chro in chrolist :
                        self.chro2len[chro] = int(length)
                        self.otherchrolist.append(chro)
            fpchrolen1.close()
            fpchrolen2 = open(self.lens2,'r', encoding='utf-8')
            for row in fpchrolen2:
                if row[0] == '#' or row == '\n':
                    continue
                chro,length = row.split('\t')[0],row.split('\t')[1]
                # chro = chro.upper()
                if len(chro) > 10 :
                    continue
                # sp = chro[:2].upper()
                sp = chro[:2]
                if self.iscompletegenome[sp] == 1 :
                    self.chro2len[chro] = int(length)
                    self.otherchrolist.append(chro)
                else:
                    if chro in chrolist :
                        self.chro2len[chro] = int(length)
                        self.otherchrolist.append(chro)
            fpchrolen2.close()





            ### full chro list

            for i in self.otherchrolist:
                self.labels.append(i)


            fpgff1 = open(self.gff1,'r', encoding='utf-8')
            for row in fpgff1:
                ch, gene, start, end = row.split()[0],row.split()[1],row.split()[2],row.split()[3]
                gene = readname(gene,self.species1+self.species2)
                start = int(start)
                end = int(end)
                self.gene2pos[gene] = int(start)
                # print(start)
                self.gene2chain[gene] = int((end-start)/abs(end -start))
            #    print gene, int(start), self.gene2chain
            fpgff1.close()
            fpgff2 = open(self.gff2,'r', encoding='utf-8')
            for row in fpgff2:
                ch, gene, start, end = row.split()[0],row.split()[1],row.split()[2],row.split()[3]
                gene = readname(gene,self.species1+self.species2)
                start = int(start)
                end = int(end)
                self.gene2pos[gene] = int(start)
                # print(start)
                self.gene2chain[gene] = int((end-start)/abs(end -start))
            #    print gene, int(start), self.gene2chain
            fpgff2.close()

        else:
            lengthset = set()
            lengthset.add(len(self.species1))
            chrolist = []


            fpchrolen1 = open(self.lens1,'r', encoding='utf-8')
            for row in fpchrolen1:
                chro = row.split('\t')[0]
                for i in lengthset:
                    if (chro[:i] in [self.species1,self.species2]):
                        chrolist.append(chro)
                    else:
                        pass
            fpchrolen1.close()

            for i in range(len(chrolist)):
                string = chrolist[i]
            #   print string[0:2]
                isnew = 1
                for sp in self.specieslist:
                    if sp == string[0:2]:
                        isnew = 0
                        break
                if isnew==1:
                    self.specieslist.append(string[0:2])
                    if string == string[0:2]:
                        self.iscompletegenome[string[0:2]] = 1
                    else:
                        self.iscompletegenome[string[0:2]] = 0

            ### input chromosome length
            fpchrolen1 = open(self.lens1,'r', encoding='utf-8')
            for row in fpchrolen1:
                if row[0] == '#' or row == '\n':
                    continue
                chro,length = row.split('\t')[0],row.split('\t')[1]
                # chro = chro.upper()
                if len(chro) > 10 :
                    continue
                # sp = chro[:2].upper()
                sp = chro[:2]
                if self.iscompletegenome[sp] == 1 :
                    self.chro2len[chro] = int(length)
                    self.otherchrolist.append(chro)
                else:
                    if chro in chrolist :
                        self.chro2len[chro] = int(length)
                        self.otherchrolist.append(chro)
            fpchrolen1.close()

            ### full chro list

            for i in self.otherchrolist:
                self.labels.append(i)

            fpgff1 = open(self.gff1,'r', encoding='utf-8')
            for row in fpgff1:
                ch, gene, start, end = row.split()[0],row.split()[1],row.split()[2],row.split()[3]
                gene = readname(gene,self.species1+self.species2)
                start = int(start)
                end = int(end)
                self.gene2pos[gene] = int(start)
                # print(start)
                self.gene2chain[gene] = int((end-start)/abs(end -start))
            #    print gene, int(start), self.gene2chain
            fpgff1.close()

        ### input gene family gene1 gene2 Ka Ks
        for row in fpgenefamilyinf:
            if (row.split()[0] == 'id1'):
                continue
            if len(row.split('\t')) < 4:
                continue
            gene1, gene2, Ka, Ks = row.split()[0],row.split()[1],row.split()[2],row.split()[3]
            gene1 = readname(gene1,self.species1+self.species2)
            gene2 = readname(gene2,self.species1+self.species2)
            if (gene1 not in self.gene2pos.keys() or gene2 not in self.gene2pos.keys()):
                continue
            genepair = ""
            if gene1 < gene2:
               genepair = gene1 + " " + gene2
            else:
               genepair = gene2 + " " + gene1
            if (gene1.split("^")[0] not in self.otherchrolist or gene2.split("^")[0] not in self.otherchrolist):
               continue
            if len(gene1.split("^")[0]) < 10  and len(gene2.split("^")[0]) < 10 :
               self.genepair2Ka[genepair] = float(Ka)
               self.genepair2Ks[genepair] = float(Ks)
            if gene1 not in self.genes:
                if len(gene1.split("^")[0]) < 10:
                         self.genes.append(gene1)
            if gene2 not in self.genes :
                if len(gene2.split("^")[0]) < 10:
                     self.genes.append(gene2)
        fpgenefamilyinf.close()
        return root

    def rad_to_coord(self, angle, radius):
        return radius*cos(angle), radius*sin(angle)

    def to_radian(self, bp, total):
        # from basepair return as radian
        # print ("to_radian", bp, total)
        return radians(bp*360./total)

    def plot_arc(self, start, stop, radius):
        # start, stop measured in radian
        t = arange(start, stop, pi/720.)
        x, y = radius*cos(t), radius*sin(t)
        plot(x, y, "k-", alpha=.5)# 染色体圆弧

    def plot_cap(self, angle, clockwise):
        radius=self.sm_radius
        # angle measured in radian, clockwise is boolean
        if clockwise: 
            t = arange(angle, angle+pi, pi/30.)
        else: 
            t = arange(angle, angle-pi, -pi/30.)
        x, y = radius*cos(t), radius*sin(t)
        middle_r = (self.radius_a+self.radius_b)/2
        x, y = x + middle_r*cos(angle), y + middle_r*sin(angle)
        plot(x, y, "k-", alpha=.5)# 边缘

    def zj(self):
        fullchrolen = int(pd.DataFrame(self.chro2len.values()).sum())
        chr_number = len(self.labels) # total number of chromosomes
        GAP = fullchrolen/self.GAP_RATIO/chr_number # gap size in base pair
        total_size = fullchrolen + chr_number * GAP # base pairs
        for i in range(chr_number):
            self.start_list.append(0)
        for i in range(1, chr_number):
            self.start_list[i] = self.start_list[i-1] + self.chro2len[self.labels[i-1]] + GAP
        stop_list = [(self.start_list[i] + self.chro2len[self.labels[i]]) for i in range(chr_number)]
        return stop_list, total_size, chr_number

    def transform_pt(self, ch, pos, r, total_size):
        rad = self.to_radian(pos + self.start_list[ch], total_size)
        return r*cos(rad), r*sin(rad)

    def plot_bez_inner(self, p1, p2, cl, total_size,alp):
    #    print "inner"
        a, b, c = p1
        ex1x, ex1y = self.transform_pt(a, b, c, total_size)
        a, b, c = p2
        ex2x, ex2y = self.transform_pt(a, b, c, total_size)
        # Bezier ratio, controls curve, lower ratio => closer to center
        ratio = .5
        x = [ex1x, ex1x*ratio, ex2x*ratio, ex2x]
        y = [ex1y, ex1y*ratio, ex2y*ratio, ex2y]
        step = .01
        t = arange(0, 1+step, step)
        xt = Bezier(x, t)
        yt = Bezier(y, t)
        # print(total_size)
        plot(xt, yt, '-', color=cl, lw=0.3, alpha = alp)#alpha 

    def run(self):
        self.radius_a = float(self.radius)
        self.radius_b = self.radius_a + 0.005
        self.sm_radius=(self.radius_b-self.radius_a)/2 #telomere capping
        root = self.ksrun()
        gene_average = (gene_length(self.gff1)+gene_length(self.gff2))/2
        stop_list, total_size, chr_number = self.zj()
        
        ## sort gene according to lacation on circle
        genessorted = []
        geneno = 0
        for i in range(len(self.genes)):
        #    print i, genes[i]

            if geneno == 0:
                genessorted.append(self.genes[0])
                geneno = geneno + 1
            else:
                firstgene = genessorted[0]
                lastgene = genessorted[-1]
                chf = firstgene.split("^")[0]
                chl = lastgene.split("^")[0]
                if (chf not in self.otherchrolist or chl not in self.otherchrolist):
                    continue
                #print firstgene, lastgene, chf, chl, self.gene2pos[firstgene]
                posf = self.gene2pos[firstgene] + self.start_list[self.labels.index(chf)]
                posl = self.gene2pos[lastgene] + self.start_list[self.labels.index(chl)]
                chi = self.genes[i].split("^")[0]
                posi = self.gene2pos[self.genes[i]] + self.start_list[self.labels.index(chi)]
        #        print posf, posl, posi
                if posi <= posf:
                    genessorted[0:0] = [self.genes[i]]
                elif posi >= posl:
                    genessorted.append(self.genes[i])
                else:
                    for j in range(len(genessorted)-1):
                        chj = genessorted[j].split("^")[0]
                        posj = self.gene2pos[genessorted[j]] + self.start_list[self.labels.index(chj)]
                        chj1 = genessorted[j+1].split("^")[0]
                        posj1 = self.gene2pos[genessorted[j+1]]+self.start_list[self.labels.index(chj1)]
                        #print posj, posj1, posi
                        if posi > posj and posi < posj1:
                            genessorted[j+1:j+1] = [self.genes[i]]
        ###########
        ###########alpha genepairs
        if self.genepairsfile_type == "famCircle":
            alphagenepairs = open(self.genepairs, 'r', encoding='utf-8')
            row = alphagenepairs.readline()#
            rowno = 0
            istoread = 0
            data1 = []
            data2 = []
            data3 = []
            data4 = []
            data5 = []
            for row in alphagenepairs:
                if (row[0] == '\n'):
                    continue
                elif (row[0] == '#'):
                    lt = row.strip('\n').split(' ')
                    block = {}
                    for i in lt:
                        if '=' in str(i):
                            lt0 = i.split('=')
                            block[str(lt0[0])] = str(lt0[1])
                    N = int(block['N'])
                    if N >= int(self.block):
                        self.class1 = True
                    else :
                        self.class1 = False
                else:
                    if self.class1:
                        seqname = []
                        for i in [self.species1,self.species2]:# 物种之间采用下划线隔开
                            seqname.append(i[:2])# 两个字符的名字
                        lt = row.strip('\n').split()
                        try :
                            if (lt[2][:2] in seqname):
                                id1, id2 = row.split()[0],row.split()[1]
                        except:
                            id1, id2 = row.split()[0],row.split()[1]
                        if (id1[1] == '*'):
                            id1 = id1[2:]
                            col = id1[0]
                        else:
                            col = 'green'
                        id1 = readname(id1,self.species1+self.species2)
                        id2 = readname(id2,self.species1+self.species2)
                        if (id1 not in self.gene2pos.keys() or id2 not in self.gene2pos.keys()):
                            continue
                        pos1 = self.gene2pos[id1]
                        pos2 = self.gene2pos[id2]
                        chro1 = id1.split("^")[0]
                        chro2 = id2.split("^")[0]
                        if (chro1 not in self.otherchrolist or chro2 not in self.otherchrolist):
                            continue
                        sp1 = chro1[0:2]
                        sp2 = chro2[0:2]
                        # print(chro1,chro2)
                        if(chro1 not in self.labels or chro2 not in self.labels):
                            continue
                        order1 = self.labels.index(chro1)
                        order2 = self.labels.index(chro2)
                        # print(abs(pos1 - pos2))
                        # print(order1, pos1, self.radius_a, order2, pos2, self.radius_a)
                        if id1.split('^')[0] == id2.split('^')[0] and abs(pos1 - pos2) < (100 * gene_average):# 同一条染色体且距离小于100
                            data1.append([order1, pos1, self.radius_a, order2, pos2, self.radius_a, 'lime', total_size,0.3])
                            # self.plot_bez_inner((order1, pos1, self.radius_a), (order2, pos2, self.radius_a), 'silver', total_size)
                            # pass
                        elif id1.split('^')[0] == id2.split('^')[0] and abs(pos1 - pos2) < (400 * gene_average):# 同一条染色体且距离小于400
                            data2.append([order1, pos1, self.radius_a, order2, pos2, self.radius_a, 'cyan', total_size,0.2])
                            # self.plot_bez_inner((order1, pos1, self.radius_a), (order2, pos2, self.radius_a), 'silver', total_size)
                            # pass
                        else:
                            data3.append([order1, pos1, self.radius_a, order2, pos2, self.radius_a, 'silver', total_size,0.1])# 其他
                            # self.plot_bez_inner((order1, pos1, self.radius_a), (order2, pos2, self.radius_a), "lime", total_size)
                            # pass
                        rowno = rowno + 1
                    else:
                        # pass
                        seqname = []
                        for i in [self.species1,self.species2]:
                            seqname.append(i[:2])
                        lt = row.strip('\n').split()
                        try :
                            if (lt[2][:2] in seqname):
                                id1, id2 = row.split()[0],row.split()[1]
                        except:
                            id1, id2 = row.split()[0],row.split()[1]
                        if (id1[1] == '*'):
                            id1 = id1[2:]
                            col = id1[0]
                        else:
                            col = 'green'
                        id1 = readname(id1,self.species1+self.species2)
                        id2 = readname(id2,self.species1+self.species2)
                        if (id1 not in self.gene2pos.keys() or id2 not in self.gene2pos.keys()):
                            continue
                        pos1 = self.gene2pos[id1]
                        pos2 = self.gene2pos[id2]
                        chro1 = id1.split("^")[0]
                        chro2 = id2.split("^")[0]
                        if (chro1 not in self.otherchrolist or chro2 not in self.otherchrolist):
                            continue
                        sp1 = chro1[0:2]
                        sp2 = chro2[0:2]
                        if(chro1 not in self.labels or chro2 not in self.labels):
                            continue
                        order1 = self.labels.index(chro1)
                        order2 = self.labels.index(chro2)
                        if id1.split('^')[0] == id2.split('^')[0] and abs(pos1 - pos2) < (100 * gene_average):# 同染色体间距小于100且block小于self.block
                            data4.append([order1, pos1, self.radius_a, order2, pos2, self.radius_a, '#b44c97', total_size,0.2])
                        else:# 同染色体间距小于100且block小于self.block
                            data5.append([order1, pos1, self.radius_a, order2, pos2, self.radius_a, '#ee7800', total_size,0.15])
                        rowno = rowno + 1
            alphagenepairs.close()
        elif self.genepairsfile_type == "wgdi":
            alphagenepairs = open(self.genepairs, 'r', encoding='utf-8')
            row = alphagenepairs.readline()#
            rowno = 0
            istoread = 0
            data1 = []
            data2 = []
            data3 = []
            data4 = []
            data5 = []
            for row in alphagenepairs:
                if (row[0] == '\n'):
                    continue
                elif (row[0] == '#'):
                    lt = row.strip('\n').split(' ')
                    block = {}
                    for i in lt:
                        if '=' in str(i):
                            lt0 = i.split('=')
                            block[str(lt0[0])] = str(lt0[1])
                    N = int(block['N'])
                    if N >= int(self.block):
                        self.class1 = True
                    else :
                        self.class1 = False
                else:
                    if self.class1:
                        seqname = []
                        for i in [self.species1,self.species2]:# 物种之间采用下划线隔开
                            seqname.append(i[:2])# 两个字符的名字
                        lt = row.strip('\n').split()
                        try :
                            if (lt[2][:2] in seqname):
                                id1, id2 = row.split()[0],row.split()[2]
                        except:
                            id1, id2 = row.split()[0],row.split()[1]
                        if (id1[1] == '*'):
                            id1 = id1[2:]
                            col = id1[0]
                        else:
                            col = 'green'
                        id1 = readname(id1,self.species1+self.species2)
                        id2 = readname(id2,self.species1+self.species2)
                        if (id1 not in self.gene2pos.keys() or id2 not in self.gene2pos.keys()):
                            continue
                        pos1 = self.gene2pos[id1]
                        pos2 = self.gene2pos[id2]
                        chro1 = id1.split("^")[0]
                        chro2 = id2.split("^")[0]
                        if (chro1 not in self.otherchrolist or chro2 not in self.otherchrolist):
                            continue
                        sp1 = chro1[0:2]
                        sp2 = chro2[0:2]
                        # print(chro1,chro2)
                        if(chro1 not in self.labels or chro2 not in self.labels):
                            continue
                        order1 = self.labels.index(chro1)
                        order2 = self.labels.index(chro2)
                        # print(abs(pos1 - pos2))
                        # print(order1, pos1, self.radius_a, order2, pos2, self.radius_a)
                        if id1.split('^')[0] == id2.split('^')[0] and abs(pos1 - pos2) < (100 * gene_average):# 同一条染色体且距离小于100
                            data1.append([order1, pos1, self.radius_a, order2, pos2, self.radius_a, 'lime', total_size,0.3])
                            # self.plot_bez_inner((order1, pos1, self.radius_a), (order2, pos2, self.radius_a), 'silver', total_size)
                            # pass
                        elif id1.split('^')[0] == id2.split('^')[0] and abs(pos1 - pos2) < (400 * gene_average):# 同一条染色体且距离小于400
                            data2.append([order1, pos1, self.radius_a, order2, pos2, self.radius_a, 'cyan', total_size,0.2])
                            # self.plot_bez_inner((order1, pos1, self.radius_a), (order2, pos2, self.radius_a), 'silver', total_size)
                            # pass
                        else:
                            data3.append([order1, pos1, self.radius_a, order2, pos2, self.radius_a, 'silver', total_size,0.1])# 其他
                            # self.plot_bez_inner((order1, pos1, self.radius_a), (order2, pos2, self.radius_a), "lime", total_size)
                            # pass
                        rowno = rowno + 1
                    else:
                        # pass
                        seqname = []
                        for i in [self.species1,self.species2]:
                            seqname.append(i[:2])
                        lt = row.strip('\n').split()
                        try :
                            if (lt[2][:2] in seqname):
                                id1, id2 = row.split()[0],row.split()[2]
                        except:
                            id1, id2 = row.split()[0],row.split()[1]
                        if (id1[1] == '*'):
                            id1 = id1[2:]
                            col = id1[0]
                        else:
                            col = 'green'
                        id1 = readname(id1,self.species1+self.species2)
                        id2 = readname(id2,self.species1+self.species2)
                        if (id1 not in self.gene2pos.keys() or id2 not in self.gene2pos.keys()):
                            continue
                        pos1 = self.gene2pos[id1]
                        pos2 = self.gene2pos[id2]
                        chro1 = id1.split("^")[0]
                        chro2 = id2.split("^")[0]
                        if (chro1 not in self.otherchrolist or chro2 not in self.otherchrolist):
                            continue
                        sp1 = chro1[0:2]
                        sp2 = chro2[0:2]
                        if(chro1 not in self.labels or chro2 not in self.labels):
                            continue
                        order1 = self.labels.index(chro1)
                        order2 = self.labels.index(chro2)
                        if id1.split('^')[0] == id2.split('^')[0] and abs(pos1 - pos2) < (100 * gene_average):# 同染色体间距小于100且block小于self.block
                            data4.append([order1, pos1, self.radius_a, order2, pos2, self.radius_a, '#b44c97', total_size,0.2])
                        else:# 同染色体间距小于100且block小于self.block
                            data5.append([order1, pos1, self.radius_a, order2, pos2, self.radius_a, '#ee7800', total_size,0.15])
                        rowno = rowno + 1
            alphagenepairs.close()
        elif self.genepairsfile_type == "ColinearScan":
            alphagenepairs = open(self.genepairs, 'r', encoding='utf-8')
            row = alphagenepairs.readline()#
            rowno = 0
            istoread = 0
            data1 = []
            data2 = []
            data3 = []
            data4 = []
            data5 = []
            for row in alphagenepairs:
                if (row[0] == '\n' or row[0] == '+'):
                    continue
                elif (row[:3] == 'the'):
                    lt = row.strip('\n').split()
                    N = int(lt[-1])
                    # block = {}
                    # for i in lt:
                    #     if '=' in str(i):
                    #         lt0 = i.split('=')
                    #         block[str(lt0[0])] = str(lt0[1])
                    # N = int(block['N'])
                    if N >= int(self.block):
                        self.class1 = True
                    else :
                        self.class1 = False
                else:
                    if self.class1:
                        seqname = []
                        for i in [self.species1,self.species2]:# 物种之间采用下划线隔开
                            seqname.append(i[:2])# 两个字符的名字
                        lt = row.strip('\n').split()
                        try :
                            if (lt[2][:2] in seqname):
                                id1, id2 = row.split()[0],row.split()[2]
                        except:
                            id1, id2 = row.split()[0],row.split()[1]
                        if (id1[1] == '*'):
                            id1 = id1[2:]
                            col = id1[0]
                        else:
                            col = 'green'
                        id1 = readname(id1,self.species1+self.species2)
                        id2 = readname(id2,self.species1+self.species2)
                        if (id1 not in self.gene2pos.keys() or id2 not in self.gene2pos.keys()):
                            continue
                        pos1 = self.gene2pos[id1]
                        pos2 = self.gene2pos[id2]
                        chro1 = id1.split("^")[0]
                        chro2 = id2.split("^")[0]
                        if (chro1 not in self.otherchrolist or chro2 not in self.otherchrolist):
                            continue
                        sp1 = chro1[0:2]
                        sp2 = chro2[0:2]
                        # print(chro1,chro2)
                        if(chro1 not in self.labels or chro2 not in self.labels):
                            continue
                        order1 = self.labels.index(chro1)
                        order2 = self.labels.index(chro2)
                        # print(abs(pos1 - pos2))
                        # print(order1, pos1, self.radius_a, order2, pos2, self.radius_a)
                        if id1.split('^')[0] == id2.split('^')[0] and abs(pos1 - pos2) < (100 * gene_average):# 同一条染色体且距离小于100
                            data1.append([order1, pos1, self.radius_a, order2, pos2, self.radius_a, 'lime', total_size,0.3])
                            # self.plot_bez_inner((order1, pos1, self.radius_a), (order2, pos2, self.radius_a), 'silver', total_size)
                            # pass
                        elif id1.split('^')[0] == id2.split('^')[0] and abs(pos1 - pos2) < (400 * gene_average):# 同一条染色体且距离小于400
                            data2.append([order1, pos1, self.radius_a, order2, pos2, self.radius_a, 'cyan', total_size,0.2])
                            # self.plot_bez_inner((order1, pos1, self.radius_a), (order2, pos2, self.radius_a), 'silver', total_size)
                            # pass
                        else:
                            data3.append([order1, pos1, self.radius_a, order2, pos2, self.radius_a, 'silver', total_size,0.1])# 其他
                            # self.plot_bez_inner((order1, pos1, self.radius_a), (order2, pos2, self.radius_a), "lime", total_size)
                            # pass
                        rowno = rowno + 1
                    else:
                        # pass
                        seqname = []
                        for i in [self.species1,self.species2]:
                            seqname.append(i[:2])
                        lt = row.strip('\n').split()
                        try :
                            if (lt[2][:2] in seqname):
                                id1, id2 = row.split()[0],row.split()[2]
                        except:
                            id1, id2 = row.split()[0],row.split()[1]
                        if (id1[1] == '*'):
                            id1 = id1[2:]
                            col = id1[0]
                        else:
                            col = 'green'
                        id1 = readname(id1,self.species1+self.species2)
                        id2 = readname(id2,self.species1+self.species2)
                        if (id1 not in self.gene2pos.keys() or id2 not in self.gene2pos.keys()):
                            continue
                        pos1 = self.gene2pos[id1]
                        pos2 = self.gene2pos[id2]
                        chro1 = id1.split("^")[0]
                        chro2 = id2.split("^")[0]
                        if (chro1 not in self.otherchrolist or chro2 not in self.otherchrolist):
                            continue
                        sp1 = chro1[0:2]
                        sp2 = chro2[0:2]
                        if(chro1 not in self.labels or chro2 not in self.labels):
                            continue
                        order1 = self.labels.index(chro1)
                        order2 = self.labels.index(chro2)
                        if id1.split('^')[0] == id2.split('^')[0] and abs(pos1 - pos2) < (100 * gene_average):# 同染色体间距小于100且block小于self.block
                            data4.append([order1, pos1, self.radius_a, order2, pos2, self.radius_a, '#b44c97', total_size,0.2])
                        else:# 同染色体间距小于100且block小于self.block
                            data5.append([order1, pos1, self.radius_a, order2, pos2, self.radius_a, '#ee7800', total_size,0.15])
                        rowno = rowno + 1
            alphagenepairs.close()
        elif self.genepairsfile_type == "MCScanX":
            alphagenepairs = open(self.genepairs, 'r', encoding='utf-8')
            row = alphagenepairs.readline()#
            rowno = 0
            istoread = 0
            data1 = []
            data2 = []
            data3 = []
            data4 = []
            data5 = []
            for row in alphagenepairs:
                if (row[0] == '\n'):
                    continue
                elif (row[:12] == '## Alignment'):
                    lt = row.strip('\n').split(' ')
                    block = {}
                    for i in lt:
                        if '=' in str(i):
                            lt0 = i.split('=')
                            block[str(lt0[0])] = str(lt0[1])
                    N = int(block['N'])
                    if N >= int(self.block):
                        self.class1 = True
                    else :
                        self.class1 = False
                elif ('#' not in row):
                    if self.class1:
                        seqname = []
                        for i in [self.species1,self.species2]:# 物种之间采用下划线隔开
                            seqname.append(i[:2])# 两个字符的名字
                        lt = row.strip('\n').split()
                        try :
                            if (lt[2][:2] in seqname):
                                id1, id2 = row.split()[2],row.split()[3]
                        except:
                            id1, id2 = row.split()[2],row.split()[3]
                        if (id1[1] == '*'):
                            id1 = id1[2:]
                            col = id1[0]
                        else:
                            col = 'green'
                        id1 = readname(id1,self.species1+self.species2)
                        id2 = readname(id2,self.species1+self.species2)
                        if (id1 not in self.gene2pos.keys() or id2 not in self.gene2pos.keys()):
                            continue
                        pos1 = self.gene2pos[id1]
                        pos2 = self.gene2pos[id2]
                        chro1 = id1.split("^")[0]
                        chro2 = id2.split("^")[0]
                        if (chro1 not in self.otherchrolist or chro2 not in self.otherchrolist):
                            continue
                        sp1 = chro1[0:2]
                        sp2 = chro2[0:2]
                        # print(chro1,chro2)
                        if(chro1 not in self.labels or chro2 not in self.labels):
                            continue
                        order1 = self.labels.index(chro1)
                        order2 = self.labels.index(chro2)
                        # print(abs(pos1 - pos2))
                        # print(order1, pos1, self.radius_a, order2, pos2, self.radius_a)
                        if id1.split('^')[0] == id2.split('^')[0] and abs(pos1 - pos2) < (100 * gene_average):# 同一条染色体且距离小于100
                            data1.append([order1, pos1, self.radius_a, order2, pos2, self.radius_a, 'lime', total_size,0.3])
                            # self.plot_bez_inner((order1, pos1, self.radius_a), (order2, pos2, self.radius_a), 'silver', total_size)
                            # pass
                        elif id1.split('^')[0] == id2.split('^')[0] and abs(pos1 - pos2) < (400 * gene_average):# 同一条染色体且距离小于400
                            data2.append([order1, pos1, self.radius_a, order2, pos2, self.radius_a, 'cyan', total_size,0.2])
                            # self.plot_bez_inner((order1, pos1, self.radius_a), (order2, pos2, self.radius_a), 'silver', total_size)
                            # pass
                        else:
                            data3.append([order1, pos1, self.radius_a, order2, pos2, self.radius_a, 'silver', total_size,0.1])# 其他
                            # self.plot_bez_inner((order1, pos1, self.radius_a), (order2, pos2, self.radius_a), "lime", total_size)
                            # pass
                        rowno = rowno + 1
                    else:
                        # pass
                        seqname = []
                        for i in [self.species1,self.species2]:
                            seqname.append(i[:2])
                        lt = row.strip('\n').split()
                        try :
                            if (lt[2][:2] in seqname):
                                id1, id2 = row.split()[2],row.split()[3]
                        except:
                            id1, id2 = row.split()[2],row.split()[3]
                        if (id1[1] == '*'):
                            id1 = id1[2:]
                            col = id1[0]
                        else:
                            col = 'green'
                        id1 = readname(id1,self.species1+self.species2)
                        id2 = readname(id2,self.species1+self.species2)
                        if (id1 not in self.gene2pos.keys() or id2 not in self.gene2pos.keys()):
                            continue
                        pos1 = self.gene2pos[id1]
                        pos2 = self.gene2pos[id2]
                        chro1 = id1.split("^")[0]
                        chro2 = id2.split("^")[0]
                        if (chro1 not in self.otherchrolist or chro2 not in self.otherchrolist):
                            continue
                        sp1 = chro1[0:2]
                        sp2 = chro2[0:2]
                        if(chro1 not in self.labels or chro2 not in self.labels):
                            continue
                        order1 = self.labels.index(chro1)
                        order2 = self.labels.index(chro2)
                        if id1.split('^')[0] == id2.split('^')[0] and abs(pos1 - pos2) < (100 * gene_average):# 同染色体间距小于100且block小于self.block
                            data4.append([order1, pos1, self.radius_a, order2, pos2, self.radius_a, '#b44c97', total_size,0.2])
                        else:# 同染色体间距小于100且block小于self.block
                            data5.append([order1, pos1, self.radius_a, order2, pos2, self.radius_a, '#ee7800', total_size,0.15])
                        rowno = rowno + 1
            alphagenepairs.close()
        else:
            print('genepairsfile_type error: File Format not recognized!')
            exit()
        for i in data3:
            self.plot_bez_inner((i[0],i[1],i[2]),(i[3],i[4],i[5]),i[6],i[7],i[8])
        # for i in data5:
        #     self.plot_bez_inner((i[0],i[1],i[2]),(i[3],i[4],i[5]),i[6],i[7],i[8])
        for i in data2:
            self.plot_bez_inner((i[0],i[1],i[2]),(i[3],i[4],i[5]),i[6],i[7],i[8])
        for i in data1:
            self.plot_bez_inner((i[0],i[1],i[2]),(i[3],i[4],i[5]),i[6],i[7],i[8])
        # for i in data4:
        #     self.plot_bez_inner((i[0],i[1],i[2]),(i[3],i[4],i[5]),i[6],i[7],i[8])

        # the chromosome layout
        j = 0
        for start, stop in zip(self.start_list, stop_list):
            start, stop = self.to_radian(start, total_size), self.to_radian(stop, total_size)
            # shaft
            self.plot_arc(start, stop, self.radius_a)
            self.plot_arc(start, stop, self.radius_b)

            # telemere capping
            clockwise=False
            self.plot_cap(start, clockwise)
            clockwise=True
            self.plot_cap(stop, clockwise)
            
            # chromosome self.labels
            label_x, label_y = self.rad_to_coord((start+stop)/2, self.radius_b*1.1)# text
            #print label_x, label_y
            text(label_x, label_y, self.labels[j], horizontalalignment="center", verticalalignment="center", fontsize = 7, color = 'black')
            j+=1
        ########
        root.set_xlim(-.8, .8)#-.5, .5
        root.set_ylim(-.8, .8)
        root.set_axis_off()
        savefig(self.savefile, dpi=1000)
        sys.exit(0)
